/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <string>
#include <map>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include "globals.h"
#include "synchronize.h"
#include "Request.h"

//#define DEBUG

using namespace std;

int main(int argc, char *argv[])
{
    if (argc != 2)
    {
        fprintf(stderr, "Usage: id\n");
        return -1;
    }

    char requester_name = argv[1][0];
    int base_length = strlen(SEMAPHORE_NAME_BASE);

#if DEBUG
    name = '#';
#endif

    // If this is Requester A, remove the old FIFO
    // and create a new one.
    if (requester_name == 'A')
    {
        unlink(FIFO_PATH);
        mkfifo(FIFO_PATH, 0666);

        printf("\n");
    }

    // Name of this requester's semaphore.
    Semaphore *sem = new Semaphore();
    char *semaphore_name = (char *)malloc(base_length + 2);
    sprintf(semaphore_name, "%s%c", SEMAPHORE_NAME_BASE, requester_name);

    // Remove the old semaphore,
    // create a new one, and open it.
    REMOVE(semaphore_name);
    printf("Removed %s\n", semaphore_name);
    printf("Created %s: %p\n", semaphore_name,
           CREATE(sem, semaphore_name, 0));
    printf("Opened %s: %p\n\n", semaphore_name, OPEN(sem, semaphore_name));

    // First open the FIFO in write-only mode.
    int fd = open(FIFO_PATH, O_WRONLY);

#ifndef DEBUG
    srand(time(0));
#else
    srand(0);
#endif

    Request req(requester_name, fd);

    // Loop to make allocation and deallocation requests.
    for (;;)
    {
        int r = rand()%10;

        if (r <= 6)
        {
            req.request_allocation();
            WAIT(sem);

            printf("fulfilled\n");
            fflush(stdout);
        }
        else req.request_deallocation();

#if DEBUG
        name++;
        if (requester_name == '.') requester_name++;
        if (requester_name == '/') requester_name++;
        if (requester_name == '0') requester_name = ':';
        if (requester_name == 'z') requester_name = '#';
        req.set_name(requester_name);
#endif

        usleep(U_SECONDS);
    }
}
